 %*****************************************************************************************
 % ROUTINE: Propeller.m
 % ----------------------------------------------------------------------------------------
 %
 % Object: This routine holds most of the functions used to calculate the
 % propeller. The routine creates a Propeller object holding all the
 % calculated velocities, forces, geometry variables defining the
 % propeller.
 %
 % Inputs:
 % Name      Description
 % ----------------------------------------------------------------------------------------
 % D        Propeller diameter
 % Z        Number of Blades
 % rBoss    Hub radius
 % RPM      Revolutions per minute
 % Uship    Ship speed in m/s
 % h        Shaft immersion
 % nrStrips   Number of strips used in calculations
 %
 % ----------------------------------------------------------------------------------------
 % ========================================================================================
 % Written by: Edvard Knutsen Skaaland
 % Last edited: 10.05.16
 % ****************************************************************************************

classdef Propeller
    properties
        D;      % Diameter
        R;      % Radius
        Z;      % Number of blades
        rBoss;  % Hub radius
        RPM;    % Revolutions per minute
        omega;  % Angular velocity
        Uship;  % Ship speed in m/s
        Lship;  % Length of ship (Only used for scaling of model wake)
        Umodel; % Model ship speed
        Lmodel; % Length of model ship
        h;      % Shaft immersion
        nrStrips; % Number of Strips used in calculations
        
        rho;    % Density of sea water
        g;      % Gravitational acceleration
        pv;     % Vapor pressure
        patm;   % Atmospheric pressure
        nu;     % Kinematic viscosity
        
        r;      % Vector of radial calculation points. 0 at center of hub
        x;      % Vector of radial calculation point. from 0 to 1 root to tip
        dr;     % length between calculation points
        c;      % Vector of sectional chord lengths
        thickness; % Vector of sectional max thickness
        Gamma;     % Vector of circulation distribution
        sigma;     % cavitation numbers
        Cl;        % Lift coefficients
        Cd;        % drag coefficients
        dL;        % section lift
        dD;        % section drag    
        dT;        % section thrust
        dQ;        % section torque
        CpMin;     % minimum pressure coefficient
        %Velocities
        Rn;        % Reynold numbers
        w;         % Radial wake
        U_A;       % Axial velocity
        U_T;       % Tangential Velocity
        Umag;      % Magnitude of sectional velocity vector
        U_A0;      % Axial velocity based on ship speed - wake
        U_T0;      % Tangential velocity based on rotational velovity
        % Induced velocities
        u_Amom;    % Induced axial velocity momentum theory
        u_Tmom;    % Induced tangential velocity momentum theory
        u_Agol;    % Induced axial velocity based on goldstein factors
        u_Tgol;    % Induced tangential velocity based on goldstein factors
        u_A;       % Induced axial velocity used in calculations
        u_T;       % Induced tangential velocity used in calculations
        % Pitch
        beta;      % angle between U_A0 and U_T0
        beta_i;    % angle axial and tangential velocities including induced
        beta_iMom; % angle based on momentum theory
        beta_iGol; % angle based on goldstein theory
        alpha_i_2D;% angle of attack for 2D section
        alpha_i;   % angle of attack 3D
        phi;       % pitch angle
        PD;        % Pitch / Diameter
        camber;    % Camber
        % Forces
        Thrust;    
        Torque;
        eta0;
        
        % Cavitation;
        cavitation;  % Cavitation string 'yes' for cav 'no' for no cav 
    end
    methods
        function obj = Propeller(D,Z,rBoss,RPM,Uship,h,nrStrips,Lship,Lmodel)
            % Environmental parameters
            obj.g = 9.81;       % Gravitational acceleration [m/s^2]
            obj.rho = 1025;     % Density of seawater [kg/m^3]
            obj.patm = 101325;  % Atmospheric pressure [Pa]
            obj.pv = 1500;      % Vapor pressure [Pa]
            obj.nu = 10e-6;     % Kinematic  viscosity [m^2/s]
            
            % Propeller characteristics
            obj.D = D;
            obj.Z = Z;
            obj.rBoss = rBoss;
            obj.RPM = RPM;
            obj.h = h;
            obj.nrStrips = nrStrips;
            obj.R = obj.D/2;
            obj.omega = 2*pi*obj.RPM/60;
            
            % Ship data
            obj.Uship = Uship;
            obj.Lship = Lship;
            obj.Lmodel = Lmodel;
            obj.Umodel = obj.Uship*sqrt(obj.Lmodel/obj.Lship);
            
            % Blade variables
            obj.dr = (obj.R-obj.rBoss)/(obj.nrStrips-1);

            obj.r = zeros(1,obj.nrStrips);
            obj.x = zeros(1,obj.nrStrips);

            for i = 1:obj.nrStrips;
               obj.r(i) = obj.rBoss+(i-1)*obj.dr;
               obj.x(i) = (obj.r(i) - obj.rBoss)/(obj.R-obj.rBoss);
            end
            
            obj.c = zeros(1,obj.nrStrips);
            % Calculating thickness distribution
            t = Thickness(obj.D,obj.Z);
            obj.thickness =t.t(obj.r);
            
            obj.Gamma = zeros(1,obj.nrStrips);
            obj.sigma = zeros(1,obj.nrStrips);
            
            obj.Cl = zeros(1,obj.nrStrips);
            obj.Cd = zeros(1,obj.nrStrips);
            obj.dL = zeros(1,obj.nrStrips);
            obj.dD = zeros(1,obj.nrStrips);
            obj.dT = zeros(1,obj.nrStrips);
            obj.dQ = zeros(1,obj.nrStrips);
            obj.CpMin = zeros(1,obj.nrStrips);
            % Velocities
            obj.Rn = zeros(1,obj.nrStrips);
            obj.w = radialWake(obj,obj.r);
            obj.U_A = zeros(1,obj.nrStrips);
            obj.U_T = zeros(1,obj.nrStrips);
            obj.Umag = zeros(1,obj.nrStrips);
            
            obj.U_A0 = obj.Uship*(ones(1,obj.nrStrips)-obj.w);
            obj.U_T0 = obj.omega*obj.r;
            
            % Induced velocities
            obj.u_Amom = zeros(1,obj.nrStrips);
            obj.u_Tmom = zeros(1,obj.nrStrips);
            
            obj.u_Agol = zeros(1,obj.nrStrips);
            obj.u_Tgol = zeros(1,obj.nrStrips);
            
            obj.u_A = zeros(1,obj.nrStrips);
            obj.u_T = zeros(1,obj.nrStrips);
            
            % Pitch
            obj.beta = atan(obj.U_A0./obj.U_T0);
            obj.beta_i = zeros(1,obj.nrStrips);
            obj.beta_iMom = zeros(1,obj.nrStrips);
            obj.beta_iGol = zeros(1,obj.nrStrips);
            
            obj.alpha_i_2D = zeros(1,obj.nrStrips);
            obj.alpha_i = zeros(1,obj.nrStrips);
            obj.phi = zeros(1,obj.nrStrips);
            obj.PD = zeros(1,obj.nrStrips);
            obj.camber = zeros(1,obj.nrStrips);
            
            % Forces
            obj.Thrust = 0;
            obj.Torque = 0;
            obj.eta0 = 0;
            
            % Cavitation
            obj.cavitation = '';
            obj.cavitationExtra1 = '';
            obj.cavitationExtra2 = '';
        end
        function obj = momentumTheoryVelocities(obj)
           % Calculate induced velocities and hydrodynamic pitch angle
           % based on momentum theory
           obj.u_Tmom = obj.Z*obj.Gamma./(2*pi*obj.r);
              
           a = 0.5;
           b = obj.U_A0;
           cc = -obj.u_Tmom.*(obj.U_T0 - 0.5.*obj.u_Tmom);
           
           obj.u_Amom = 0.5.*((-b+sqrt(b.^2-4.*a.*cc))./(2*a));
           obj.beta_iMom = atan2(obj.U_A0+obj.u_Amom./2, obj.U_T0-obj.u_Tmom./2);
        end
        function obj = goldsteinFactorVelocities(obj)
            % Calculate velocities based on goldstein factors
           xx = obj.r./obj.R;
           Xi = zeros(1,obj.nrStrips);
           for i = 1:obj.nrStrips
               Xi(i) = goldsteinFactor(xx(i),obj.beta_iMom(i));
           end
           obj.u_Tgol = obj.u_Tmom./Xi;
           obj.u_Agol = obj.u_Amom./Xi;
           
           obj.beta_iGol = atan2(obj.U_A0+obj.u_Agol./2, obj.U_T0-obj.u_Tgol./2);
        end
        function obj = calculateVelocities(obj)
            % Calculate induced velocities
            obj = obj.momentumTheoryVelocities();
            % Update based on Goldstein factors
            obj = obj.goldsteinFactorVelocities();
            % Change actual velocities to Golstein
            obj.u_T = obj.u_Tgol;
            obj.u_A = obj.u_Agol;
            obj.U_T = obj.U_T0 - obj.u_T./2;
            obj.U_A = obj.U_A0 + obj.u_A./2;
            
            obj.Umag = sqrt(obj.U_T.^2+obj.U_A.^2);
            obj.beta_i = atan2(obj.U_A, obj.U_T);
            
        end
        function obj = calculateForces(obj)
            % Calculate lift
            obj.dL = obj.rho.*obj.Gamma.*obj.Umag;
            
            % Calculate lift coefficient
            obj.Cl = obj.dL./(0.5*obj.rho.*obj.c.*obj.Umag.^2);
            obj.Cl(1) = 2*obj.Cl(2) - obj.Cl(3);
            obj.Cl(end) = 2*obj.Cl(end-1) - obj.Cl(end-2);
            
            % Calculate drag coefficient
            obj.Rn = obj.Umag.*obj.c./obj.nu;
            
            obj.Cd = dragCoef(obj.Rn, obj.Cl, obj.thickness./obj.c);
            obj.Cd(1) = 2*obj.Cd(2) - obj.Cd(3);
            obj.Cd(end) = 2*obj.Cd(end-1) - obj.Cd(end-2);
            
            % Calculate drag
            obj.dD = 0.5*obj.rho*obj.Cd.*obj.c.*obj.Umag.^2;
            
            % Calculate thrust
            obj.dT = obj.rho.*obj.Gamma.*obj.U_T - obj.dD.*sin(obj.beta_i);
            
            % Calculate torque
            obj.dQ = obj.rho.*obj.Gamma.*obj.U_A.*obj.r + obj.dD.*cos(obj.beta_i);
            
            % Integrate to find global forces
            obj.Thrust = obj.Z.*trapz(obj.r,obj.dT);
            obj.Torque = obj.Z.*trapz(obj.r,obj.dQ);
            
            obj.eta0 = mean(obj.Thrust.*obj.U_A0./(obj.omega.*obj.Torque));
        end
        function obj = cavitationCheck(obj, cavsafe_f)
           % Cavitation check with radial wake
           obj.sigma = (obj.patm - obj.pv + obj.rho.*obj.g*(obj.h-obj.r))./(0.5.*obj.rho.*obj.Umag.^2);
           obj.CpMin = minPress(obj.Cl, obj.thickness./obj.c);
           obj.CpMin(1) = 2*obj.CpMin(2) -obj.CpMin(3);
           obj.CpMin(end) = 2*obj.CpMin(end-1) - obj.CpMin(end-2);
  
           % Check for radial cavitation
           if all(-obj.CpMin <= cavsafe_f.*obj.sigma)
               obj.cavitation = 'False';
           else
               obj.cavitation = 'True';
           end
        end  
        function obj = calculateGeometricVariables(obj)
            % Calculate Geometric variables
            obj.alpha_i_2D = idealAngle(obj.Cl);
            
            % 3D correction factors
            xx = obj.r./obj.R;
            k_c = 1.6946 + 0.5048.*xx - 4.0012.*xx.^2 + 4.3283.*xx.^3;
            k_a = 1 + 1.46.*xx.^3;
            k_t = 2.5*obj.Z.*obj.c.*cos(obj.beta_i)./(obj.D.*xx);
            % camber corrections
            obj.camber = k_c.*maxCamber(obj.Cl);
            
            a_i3 = obj.alpha_i_2D.*k_a;
            a_t = k_t.*obj.thickness./obj.c;
            
            obj.alpha_i = a_i3 + a_t;
            % Pitch Angle
            obj.phi = obj.alpha_i.*pi/180 + obj.beta_i;
            % Pitch/Diameter
            obj.PD = 2*pi.*obj.r.*tan(obj.phi)./obj.D;
        end
    end    
end