% This MATLAB class finds the optimum chord and circulation distribution
% by running through a^2 * m^2 possible forms
classdef optimizeChordAndCirc
   properties
       Treq; % Required thrust
       m;    % form function parameter
       mNr;  % number of m parameters
       a;    % form function parameter
       aNr;  % number of a parameters
       mGrid;% grid of m parameters
       aGrid;% grid of a parameters
       eta0; % Propeller efficiency
       thrust;     % calculated thrust
       cavitation; % cavitation
       k;    % form function parameter
       prop; % propeller object
       eta0_max; % highest achieved efficiency
       m_max; % m parameter for highest efficiency
       a_max; % a parameter for highest efficiency
       k_max; % k parameter for highest efficiency
       c;     % chamber form function parameter
       k_circ; % k parameter for circulation
       a_circ; % a parameter for circulation
       m_circ; % m parameter for circulation
       k_circMax; % k parameter for highest efficiency
       a_circMax; % a parameter for highest efficiency
       m_circMax; % m parameter for highest efficiency
   end
   methods
       function obj = optimizeChordAndCirc(c,a,m,prop,Treq) 
            obj.Treq = Treq;
            obj.m = m;
            obj.mNr = length(obj.m);
            
            obj.a = a;
            obj.aNr = length(obj.a);
            
            obj.c = c;
            
            [obj.mGrid,obj.aGrid] = meshgrid(m,a);
            
            
            obj.eta0 = zeros(obj.aNr,obj.mNr);
            obj.thrust = zeros(obj.aNr,obj.mNr);
            obj.cavitation = zeros(obj.aNr,obj.mNr);
            obj.k_circ = zeros(obj.aNr,obj.mNr);
            obj.a_circ = zeros(obj.aNr,obj.mNr);
            obj.m_circ = zeros(obj.aNr,obj.mNr);
            obj.prop = prop;
            obj.eta0_max = 0;
            obj.m_max = 0;
            obj.a_max = 0;
            obj.k_max = 0;
            obj.k_circMax = 0;
            obj.a_circMax = 0;
            obj.m_circMax = 0;
       end
       function obj = optimize(obj,c_0,print)
           for i=1:obj.aNr
               for j = 1:obj.mNr
                   prop1 = obj.prop;
                   a1 = obj.a(i);
                   m1 = obj.m(j);
                   disp(['Analyzing a = ',num2str(a1),' and m = ',num2str(m1)]);
                   prop1.c = chordLength(prop1.x,c_0,obj.c,a1,m1);
                   
                   opt = optimizeCirculation(obj.a,obj.m,prop1,obj.Treq);
                   opt = opt.optimize(print);
                   obj.eta0(i,j) = opt.eta0_max;
                   obj.a_circ(i,j) = opt.a_max;
                   obj.m_circ(i,j) = opt.m_max;
                   obj.k_circ(i,j) = opt.k_max;
               end
           end
           if numel(find(obj.eta0))>0
               
               [maxval,maxloc] = max(obj.eta0(:));
               [maxval_row,maxval_col] = ind2sub(size(obj.eta0),maxloc);
               obj.eta0_max = maxval;
               obj.m_max = obj.m(maxval_col);
               obj.a_max = obj.a(maxval_row);
               
               obj.k_circMax = obj.k_circ(maxval_row,maxval_col);
               obj.a_circMax = obj.a_circ(maxval_row,maxval_col);
               obj.m_circMax = obj.m_circ(maxval_row,maxval_col);
           else
               obj.eta0_max = 0;
               obj.m_max = 0;
               obj.a_max = 0;
               obj.k_circMax = 0;
               obj.a_circMax = 0;
               obj.m_circMax = 0;
           end
           disp(['Optimal chord length (a,m): ',num2str([obj.a_max,obj.m_max])]);
           disp(['Optimal circulation distribution (k,a,m): ', num2str([obj.k_circMax,obj.a_circMax,obj.m_circMax])]);
           disp(['eta_0 = ', num2str(obj.eta0_max)]);
       end
   end
end