% This MATLAB class finds the optimum circulation distribution
% based on the propeller geometry
classdef optimizeCirculation
   properties
       Treq; % Required thrust
       m;    % form function parameter
       mNr;  % number of m parameters
       a;    % form function parameter
       aNr;  % number of a parameters
       mGrid;% grid of m parameters
       aGrid;% grid of a parameters
       eta0; % Propeller efficiency
       thrust;     % calculated thrust
       cavitation; % cavitation
       k;    % form function parameter
       prop; % propeller object
       eta0_max; % highest achieved efficiency
       m_max; % m parameter for highest efficiency
       a_max; % a parameter for highest efficiency
       k_max; % k parameter for highest efficiency
   end
   methods
       function obj = optimizeCirculation(a,m,prop,Treq) 
            obj.Treq = Treq;
            obj.m = m;
            obj.mNr = length(obj.m);
            
            obj.a = a;
            obj.aNr = length(obj.a);
            
            [obj.mGrid,obj.aGrid] = meshgrid(m,a);
            %obj.aGrid = meshgrid(m,a);
            
            obj.eta0 = zeros(obj.aNr,obj.mNr);
            obj.thrust = zeros(obj.aNr,obj.mNr);
            obj.cavitation = zeros(obj.aNr,obj.mNr);
            obj.k = zeros(obj.aNr,obj.mNr);
            
            obj.prop = prop;
            obj.eta0_max = 0;
            obj.m_max = 0;
            obj.a_max = 0;
            obj.k_max = 0;
       end
       function obj = optimize(obj,print)
           for i=1:obj.aNr
               for j = 1:obj.mNr
                   prop1 = obj.prop;
                   a1 = obj.a(i);
                   m1 = obj.m(j);
                   if print == 1;
                       disp(['Analyzing a = ',num2str(a1),' and m = ',num2str(m1)]);
                   end
                   % Adjust k to get necessary thrust
                   thrust_k = kThrust(prop1, a1, m1);
                   %disp(obj.Treq)
                   
                   k1 = thrust_k.kAdjust(obj.Treq,7);
                   
                   if k1~= 0
                       % set circulation distribution
                       prop1.Gamma = gammaCalc(prop1.r./prop1.R,k1,a1,m1);
                       % calculate velocities
                       prop1 = prop1.calculateVelocities();
                       % calculate forces
                       prop1 = prop1.calculateForces();
                       % check for cavitation
                       prop1 = prop1.cavitationCheck(0.8);
                       
                       obj.eta0(i,j) = prop1.eta0;
 
                       obj.thrust(i,j) = prop1.Thrust;
                       obj.k(i,j) = k1;
                       % Check for cavitation
                       if strcmp(prop1.cavitation,'True')
                           % If true
                           obj.cavitation(i,j) = 1;
                           obj.eta0(i,j) = 0; % Set efficiency to zero 
                       end
                   else
                       
                       obj.eta0(i,j) = 0;
                       obj.k(i,j)    = 0;
                       obj.thrust(i,j) = 0;
                           
                   end
                   
               end
           end 
           if numel(find(obj.eta0))>0
               
               [maxval,maxloc] = max(obj.eta0(:));
               [maxval_row,maxval_col] = ind2sub(size(obj.eta0),maxloc);
               
               obj.eta0_max = maxval;
               obj.m_max = obj.m(maxval_col);
               obj.a_max = obj.a(maxval_row);
               obj.k_max = obj.k(maxval_row,maxval_col);
           else
               obj.eta0_max = 0;
               obj.m_max = 0;
               obj.a_max = 0;
               obj.k_max = 0;
           end
           if print == 1
               disp(['Optimal circulation distribution (k, a, m) = ', num2str([obj.k_max, obj.a_max, obj.m_max])])
           end
       end
   end    
end